module SDL
  class TTF
    def self.load(path, size = 18)
      open(path, size, nil)
    end

    def write(surface, string, x, y, color = [255, 255, 255])
      draw_blended_utf8(surface, string, x, y, 
                        color[0], color[1], color[2])
    end

    def char_width
      text_size("A").first
    end

    def text_width(string)
      text_size(string).first
    end

    def text_height(string)
      text_size(string).last
    end
    
    #line_skip exists for TTF
  end

  class BMFont
    def self.load(path, size = nil)
      open(path, SDL::BMFont::TRANSPARENT)
    end

    def self.load_sfont(path, size = nil)
      open(path, SDL::BMFont::TRANSPARENT | SDL::BMFont::SFONT)
    end

    def write(surface, string, x, y, color = nil)
      set_color(*color) if color
      textout(surface, string, x, y)
    end

    #TODO: support sfont
    def text_width(string)
      text_size(string).first
    end

    def text_height(string)
      text_size(string).last
    end

    def char_width
      (width == 0) ? text_width("A") : width
    end

    alias line_skip  height
  end

  class Kanji
    def self.load(path, size = 18)
      open(path, size)
    end

    def write(surface, string, x, y, color = nil)
      put(surface, string, x, y, 
          color[0], color[1], color[2])
    end

    alias char_width  width
    alias text_width  textwidth
    alias text_height height
    alias line_skip   height
  end
end

module PumpkinModern

  class Font

    def self.load(path, size = nil, type = nil)
      type ||= File.extname(path).downcase[1..-1].to_sym
      case type
      when :ttf
        size ? SDL::TTF.load(path, size) : SDL::TTF.load(path)
      when :bmp, :bmfont
        SDL::BMFont.load(path, size)
      when :sfont
        SDL::BMFont.load_sfont(path, size)
      when :bdf, :kanji
        size ? SDL::Kanji.load(path, size) : SDL::Kanji.load(path)
      else
        raise ArgumentError, "unknown font type: #{type}"
      end
    end

  end

end
